# -*- coding: utf-8 -*-
from django.contrib.auth import models as am
from django.core.exceptions import ValidationError
from django.test import TestCase

from contacts import models as cm
from contacts.tests.utils import mfty, cfty
from references import models as rm


class ValStructure(TestCase):

    def setUp(self):
        self.usr1 = am.User.objects.create_user('usr1')
        self.fjur = rm.FormeJuridique()
        self.fjur.save()
        self.i_pm = {
            'date_creation': '2014-10-01',
            'date_derniere_modification': '2014-10-02',
            'utilisateur_derniere_modification': self.usr1,
            'f_juridique': self.fjur,
            'designation': "test",
            }
        self.i_p = {
            'nom': 'testin', 
            'prenom': 'testons', 
            'date_creation': '2014-10-01',
            'date_derniere_modification': '2014-10-02',
            'utilisateur_derniere_modification': self.usr1,
            }
        self.activite = rm.TypeActivite()


    def test_structure_bio(self):
        """Si un type d'activite bio est renseigne, la personne morale
        - doit etre un agriculteur bio"""
        # Une structure avec une activite bio qui n'est pas un agriculteur bio avec erreur
        structure = mfty(cm.Structure, self.i_pm, type_activite=self.activite)
        structure.type_pm = 1
        self.assertRaises(ValidationError, structure.full_clean)
        # Une structure sans activite bio qui est un agriculteur bio avec erreur
        structure = mfty(cm.Structure, self.i_pm, type_pm = 2)
        self.assertRaises(ValidationError, structure.full_clean)
        # Une structure avec une activite bio qui est un agriculteur bio sans erreur
        structure.type_activite = self.activite
        structure.full_clean()

    def test_structure_agri(self):
        """Si un type d'activite principale est renseigne, la personne morale
        - doit etre un agriculteur"""
        # Une structure avec une activite principale qui n'est pas un agriculteur avec erreur
        structure = mfty(cm.Structure, self.i_pm, type_activite_pri=self.activite)
        structure.type_pm = 0
        self.assertRaises(ValidationError, structure.full_clean)
        # Une structure avec une activite principale qui est un agriculteur sans erreur
        structure.type_pm = 1
        structure.full_clean()

    def test_structure_acti(self):
        """Si la personne morale est un agriculteur alors il
        - doit avoir une activite bio
        OU
        - doit avoir une activite principale"""
        # Un agriculteur qui n'a pas d'activite avec erreur
        structure = mfty(cm.Structure, self.i_pm, type_pm = 1)
        self.assertRaises(ValidationError, structure.full_clean)
        # Un agriculteur qui a une activite principale sans erreur
        structure.type_activite_pri=self.activite
        structure.full_clean()
        # Un agriculteur (bio) qui a une activite bio sans erreur
        structure = mfty(cm.Structure, self.i_pm, type_pm = 2)
        structure.type_activite=self.activite
        structure.full_clean()
    
    def test_structure_f_juridique(self):
        """Tout type de structure doit avoir une forme juridique"""
        # Une structure sans forme juridique avec erreur
        structure = mfty(cm.Structure, self.i_pm, type_pm = 0)
        structure.f_juridique = None
        self.assertRaises(ValidationError, structure.full_clean)
        # Une structure avec forme juridique sans erreur
        structure.f_juridique = self.fjur
        structure.full_clean()
        
    def test_structure_etiquette(self):
        """L'etiquette sous forme de liste doit contenir la designation
        avec les & remplacés par &amp, chaque ligne de l'adresse et 
        l'association du code postal et de la ville"""
        # Une structure avec adresse
        structure = cfty(cm.Structure, self.i_pm, type_pm = 0)
        structure.designation = None
        structure.adresse = "62 rue des \n Cacahuètes"
        etiquette = structure.contenu_etiquette()
        self.assertEqual(etiquette, ["Structure (#1)", "62 rue des ",
                                    " Cacahuètes", " "])
        # Une structure avec personne et adresse
        personne = cfty(cm.Personne, self.i_p, structure = structure)
        etiquette = structure.contenu_etiquette()
        self.assertEqual(etiquette, ["testons testin", "62 rue des ",
                                    " Cacahuètes", " "])
        # Une structure avec designation et adresse
        structure.designation = "test&testin"
        etiquette = structure.contenu_etiquette()
        self.assertEqual(etiquette, ["test&amp;testin", "62 rue des ", 
                                    " Cacahuètes", " "])
        # Une structure avec designation, adresse, code postal et ville
        structure.ville = "Canarville"
        structure.code_postal = "10000"
        etiquette = structure.contenu_etiquette()
        self.assertEqual(etiquette, ["test&amp;testin", "62 rue des ", 
                                    " Cacahuètes", "10000 Canarville"])
    
    def test_structure_liste_contact(self):
        """La méthode liste_contacts_associes doit retourner la liste
        des personnes associées"""
        # Une structure sans personne associée
        structure = cfty(cm.Structure, self.i_pm, type_pm = 0)
        liste_contact = structure.liste_contacts_associes()
        self.assertEqual(len(liste_contact), 0)
        # Une structure avec une personne associée
        personne1 = mfty(cm.Personne, self.i_p, structure = structure)
        personne1.nom = "test1"
        personne1.save()
        liste_contact = structure.liste_contacts_associes()
        self.assertEqual(len(liste_contact), 1)
        self.assertEqual(liste_contact[0].nom, "test1")
        # Une structure avec deux personnes associées
        personne2 = mfty(cm.Personne, self.i_p, structure = structure)
        personne2.nom = "test2"
        personne2.save()
        liste_contact = structure.liste_contacts_associes()
        self.assertEqual(len(liste_contact), 2)
        self.assertEqual(liste_contact[1].nom, "test2")
        
    def test_structure_adresse_electronique(self):
        """La méthode adresse_electronique doit retourner le nom de la 
        structure et son adresse electronique, ou rien si aucune
        adresse email n'est renseignée"""
        # Une structure sans adresse electronique
        structure = mfty(cm.Structure, self.i_pm, type_pm = 0)
        adresse_elec = structure.adresse_electronique()
        self.assertEqual(adresse_elec, ())
        # Une structure avec adresse electronique
        structure.courriel = "structure@test.com"
        adresse_elec = structure.adresse_electronique()
        self.assertEqual(adresse_elec, ("test", "structure@test.com"))
    
    def test_structure_get_courriels(self):
        """La méthode get_courriels doit retourner l'adresse email de
        la structure et de toutes les personne qui y sont associées
        sauf s'ils ne veulent pas recevoir d'e-mail"""
        # Une structure sans personne associée et sans adresse email
        structure = cfty(cm.Structure, self.i_pm, type_pm = 0)
        courriels = structure.get_courriels()
        self.assertEqual(courriels, [])
        # Une structure sans personne associée et avec adresse email
        structure.courriel = "structure@test.com"
        courriels = structure.get_courriels()
        self.assertEqual(courriels, [("test", "structure@test.com")])
        # Une structure avec personnes associées et avec adresse email
        personne1 = mfty(cm.Personne, self.i_p, structure = structure, 
                                                nom = "personne1")
        personne1.courriel = "personne1@test.com"
        personne1.pas_de_courriel = False
        personne1.save()
        personne2 = mfty(cm.Personne, self.i_p, structure = structure, 
                                                nom = "personne2")
        personne2.courriel = "personne2@test.com"
        personne2.pas_de_courriel = False
        personne2.save()
        courriels = structure.get_courriels()
        self.assertEqual(courriels, [("test", "structure@test.com"), 
                                    ("testons personne1", "personne1@test.com"),
                                    ("testons personne2", "personne2@test.com")])

# EOF
