# -*- coding: utf-8 -*-
#
# Ce fichier regroupe les classes et fonctions annexes utilisées dans les vues
# de l'application.
#
##
import codecs

import reportlab
import reportlab.lib.styles as rlstyles
import reportlab.platypus as rlp
from reportlab.lib.units import cm

from django.conf import settings
from django.core import exceptions

import model_walk


PAGE_HEIGHT=reportlab.rl_config.defaultPageSize[1]
PAGE_WIDTH=reportlab.rl_config.defaultPageSize[0]

REPR_MTX = {
    'ManyRelatedManager': lambda x:
        ', '.join([unicode(elm) for elm in x.all()]),
    'RelatedManager': lambda x:
        ', '.join([unicode(elm) for elm in x.all()]),
    'bool': lambda x: x and 'VRAI' or 'FAUX',
    'date': lambda x: x.strftime(settings.FORMAT_DATE_EXPORT),
    'datetime': lambda x: x.strftime(settings.FORMAT_DATE_EXPORT),
    'NoneType': lambda x: u"",
    }

def repr_obj(obj):
    class_name = type(obj).__name__
    return REPR_MTX.get(class_name, lambda x: x)(obj)

### ---------------------------------------------------------------------------
# Étiquettes en PDF
#

class EtiquettesPDF:
    """
    Classe permettant la génération simple de pages d'étiquettes d'adresses
    pour les modèles fournissant la méthode contenu_etiquette() retournant
    la liste des lignes constituant l'étiquette.
    """
    # Marges de la page
    margin_top = 0*cm
    margin_left = 0*cm
    margin_right = 0*cm
    margin_bottom = 0*cm
    # Dimensions des étiquettes (représentées par des frames)
    frame_width = 7.00*cm
    frame_height = 3.71*cm
    # Nombre de colonnes et lignes de frames par page
    frames_cols = int((PAGE_WIDTH-margin_left-margin_right)/frame_width)
    frames_rows = int((PAGE_HEIGHT-margin_top-margin_bottom)/frame_height)
    # Nombre de frames (donc d'étiquettes) par page
    fpp = frames_cols*frames_rows
    # Bricolage sur la hauteur des étiquettes pour régler un problème
    # sur l'étiquette du bas qui est trop haute sur les feuilles d'étiquettes
    # du Gabnor. (Fait ici pour ne pas impacter fpp)
	# On rajoute cette hauteur à chaque étiquette.
    frame_height += .04*cm
    # Le padding des frames, dans l'ordre demandé par rlp.Frame(),
    # c'est à dire left, bottom, right, top
    frame_padding = (10, 5, 5, 5)
    # Dessiner les bords des frames ?
    show_frame_boundary = 0
    # Première ligne des frames
    top_y = PAGE_HEIGHT-margin_top
    titre = u"Étiquettes"
    auteur = u""

    def __init__(self, file):
        """
        Calcule quelques informations avant sur le document à générer.
        """
        # Le fichier (ou l'objet émulant un fichier) où écrire
        self.file = file
        # Style des étiquettes
        self.style = rlstyles.getSampleStyleSheet()["Normal"]
        self.style.fontName = "Helvetica"
        self.style.fontSize = 11

    def build(self, objets):
        """
        Génère le document et l'écrit dans self.file.
        """
        # Création du Canvas
        self.canv = reportlab.pdfgen.canvas.Canvas(self.file)
        self.canv.setTitle(self.titre)
        self.canv.setAuthor(self.auteur)
        # Get on with it !
        page_frames = None
        for i in range(0, len(objets)):
            if i % self.fpp == 0: # On entame une nouvelle page
                if i != 0: # On génère la page qui vient d'être remplie
                    self.canv.showPage()
                # On crée la liste des frames de la nouvelle page
                page_frames = self._mk_page_frames(i//self.fpp)
            # On écrit l'étiquette dans la frame en cours
            page_frames[i % self.fpp].addFromList(
                    self._contenu_etiquette(objets[i]), self.canv)
        # Génération de la dernière page
        self.canv.showPage()
        # Génération du document
        self.canv.save()

    def _mk_page_frames(self, page_id):
        """
        Retourne une liste de frames pour une page.
        .
        page_id est le numéro de la page permettant d'assigner une identifiant
        unique à chaque frame, composé du numéro de page et du numéro de la
        frame dans cette page (même si ça n'est pas vraiment utilisé ensuite!).
        """
        frames = []
        for i in range(0, self.fpp):
            # remarque: *self.frame_padding ne fonctionne pas sous python2.5
            f = rlp.Frame(
                      self.margin_left+(i%self.frames_cols*self.frame_width),
                      self.top_y-(i/self.frames_cols+1)*self.frame_height,
                      self.frame_width,
                      self.frame_height,
                      self.frame_padding[0],
                      self.frame_padding[1],
                      self.frame_padding[2],
                      self.frame_padding[3],
                      id="%s%s" % (page_id, i),
                      showBoundary=self.show_frame_boundary
                      )
            frames.append(f)
        return frames

    def _contenu_etiquette(self, objet):
        """
        Retourne sous forme de listes de paragraphes le texte d'une étiquette
        qui sera écrit dans une frame.
        """
        return [ rlp.Paragraph(ligne, self.style)
                 for ligne in objet.contenu_etiquette() if ligne ]



### ---------------------------------------------------------------------------
# Base pour l'export CSV
#

class ExportCSV(object):
    """
    base class for building CSV file
    - subclass must define 'attr_liste', list of composed_name
       (name of attribute in a django fashion: 'attr1__attr2')
    -  output is generated to a file (that may be a StringIO)
    """
    # separator character
    SEP = u";"
    # insert utf-8 BOM should help excel understand it is utf-8...
    insert_bom = True
    attr_liste = None

    def __init__(self, file):
        "record output to self"
        self.out = file

    def _quote_csv(self, value):
        'escape charater like "(double-quote) and \r\n (carriage return)'
        if value.find('"'):
            value = value.replace('"','""')
        # Excel termine la ligne lorsqu'une nouvelle ligne est présente
        # dans un champ, même entre guillements. Tentative de contournement
        # (non testée) en remplaçant \r\n par \n 
        # voir http://stackoverflow.com/questions/1241220/
        #  generating-csv-file-for-excel-how-to-have-a-newline-inside-a-value
        if value.find('\r\n'):
            value = value.replace('\r\n','\r')
        return '"%s"' % value

    def _get_repr(self, obj, composed_name):
        """ return a representation given "obj" and
        "composed_name" with composed_name in django
        fashion: 'attr1__func__attr2'
        """
        cur_obj = obj
        for attr_name in composed_name.split('__'):
            cur_obj = getattr(cur_obj, attr_name, None)
            if callable(cur_obj):
                cur_obj = cur_obj()
            if cur_obj is None:
                break
        return self._quote_csv(unicode(repr_obj(cur_obj)))

    def _ecrit_champs(self, objet):
        """
        Écrit les champs de l'objet donné dans le fichier de sortie.
        L'objet doit posséder un attribut (désigné par notre attribut
        de classe attr_liste) qui donne la liste des champs à inclure
        dans l'export.
        """
        ligne = self.SEP.join([ self._get_repr(objet, champ)
                                for champ in self.attrs_export ])
        self.out.write((ligne+u"\r\n").encode('utf8'))

    def _get_verbose_names(self, model, list_name):
        """Return csv header given a class model (not a model intance)
        and list of a tuple of composed_name composed_name is
        separated by '__' in a django fashion: 'attr1__attr2__func'
        """
        titres = []
        for name in list_name:
            titres.append(model_walk.label_for_composed_field(model, name))
        return self.SEP.join(titres)

    def generer(self, object_list):
        """
        Génère le fichier CSV à partir d'une liste d'objets.
        .
        Note: pour les structures, la liste des attributs à exporter
        est systématiquement celle du modèle AgriculteurBio et les
        champs non trouvés seront exportés vides (ceci permet
        d'exporter en même temps tous les objets Structure,
        Agriculteur, ou AgriculteurBio).
        """
        if len(object_list) == 0:
            return
        bmodel = type(object_list[0])
        try:
            self.attrs_export = getattr(bmodel, self.attr_liste)
        except AttributeError:
            raise exceptions.ImproperlyConfigured(
                u"Le modèle %s ne possède pas l'attribut %s"
                u" fournissant la liste des champs à exporter"
                % (bmodel, self.attr_liste))
        titres = self._get_verbose_names(bmodel, self.attrs_export)
        if self.insert_bom:
            self.out.write(codecs.BOM_UTF8)
        self.out.write((titres+u"\r\n").encode('utf-8'))
        for obj in object_list:
            self._ecrit_champs(obj)


### ---------------------------------------------------------------------------
# Export CSV complet
#

class ExportCSVComplet(ExportCSV):
    """
    Classe effectuant un export CSV des attributs listés dans l'attribut de
    classe « attrs_export » du modèle.
    """
    # L'attribut du modèle fournissant la liste des attributs à exporter.
    attr_liste = 'attrs_export'



### ---------------------------------------------------------------------------
# Export CSV pour publipostage
#

class ExportCSVPublipostage(ExportCSV):
    """
    Classe effectuant un export CSV des attributs listés dans l'attribut de
    classe « attrs_publipostage » du modèle.
    """
    # L'attribut du modèle fournissant la liste des attributs à exporter.
    attr_liste = 'attrs_publipostage'

# EOF
